/**
 * Copyright (C) 2007-2013 Lawrence Murray
 *
 * This program is free software; you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the Free
 * Software Foundation; either version 2 of the License, or (at your option)
 * any later version.
 * 
 * @author Lawrence Murray <lawrence@indii.org>
 * $Rev$
 * $Date$
 */
#ifndef INDII_ILLUM_GUI_ILLUMWINDOW_HPP
#define INDII_ILLUM_GUI_ILLUMWINDOW_HPP

#include "IllumDocument.hpp"
#include "IllumGlobalPane.hpp"
#include "../model/IllumModel.hpp"
#include "../../image/ImageResource.hpp"
#include "../../gui/Window.hpp"
#include "../../gui/WindowObserver.hpp"
#include "../../gui/PreviewPane.hpp"
#include "../../gui/ChannelMixerPane.hpp"

#include "wx/frame.h"
#include "wx/aui/aui.h"

namespace indii {
/**
 * Main window.
 */
class IllumWindow : public Window {
public:
  /**
   * Constructor.
   *
   * @param parent Parent.
   * @param mode Mode of operation.
   * @param obs Enclosing application, if not embedded mode.
   */
  IllumWindow(wxWindow* parent, const Mode mode = EMBED,
      WindowObserver* obs = NULL);
  
  /**
   * Constructor.
   *
   * @param parent Parent.
   * @param res Image resource.
   * @param model Cluster model.
   * @param mode Mode of operation.
   * @param obs Enclosing application, if not embedded mode.
   */
  IllumWindow(wxWindow* parent, ImageResource* res, IllumModel* model,
      const Mode mode = EMBED, WindowObserver* obs = NULL);

  /**
   * Constructor.
   *
   * @param parent Parent.
   * @param path Path of image to open.
   * @param mode Mode of operation.
   * @param obs Enclosing application, if not embedded mode.
   */
  IllumWindow(wxWindow* parent, const wxString& path, const Mode = STANDALONE,
      WindowObserver* obs = NULL);

  virtual Document* open(const wxString& file);
  virtual void layoutDefault();
  virtual void setModel(Model* model);
  virtual bool isShown(const int i);
  virtual void show(const int i, const bool show);

protected:
  virtual void initPanes();
  using Window::enable;
  virtual void enable(const bool enable);

private:
  /**
   * Enclosing application.
   */
  WindowObserver* obs;

  /*
   * Panes.
   */
  IllumGlobalPane* globalPane;
  ChannelMixerPane* mixerPane;

  /**
   * Ids of messages displayed in the info bar.
   */
  enum {
    /**
     * Instructions on how to use mouse interface.
     */
    MSG_MOUSE_INTERFACE
  };

  DECLARE_EVENT_TABLE()
};
}

#endif
